-- Time-stamp: <Tue Dec 20 2005 13:00:45 hwloidl>
-- $Id: PPIntm.hs,v 1.1 2006/01/17 10:29:49 hwloidl Exp $
--
-- Pretty printing of AST tree for the parsed Intm language file.
---------------------------------------------------------------------------

module PPIntm where

import AbsSyn
import Text.PrettyPrint

dot      = text "."
ass     = text "<-"

render = Text.PrettyPrint.render

class Renderable a where
 pp :: a -> Doc
 
instance Renderable Exp where
 pp (CONST n) = text (show n)
 pp (NAME n) = ppString n
 pp (TEMP t) = ppString t
 pp (BINOP op l r) = pp l <> pp op <> pp r
 pp (MEM a) = text "MEM" <+> pp a
 pp (CALL f as) = text "CALL" <+> pp f <> lparen <> pp as <> rparen
 pp (ESEQ stm exp) = lbrace <> pp stm <> comma <> pp exp <> rbrace

instance Renderable Stm where
 pp (MOVE d s) = pp d <+> ass <+> pp s
 pp (EXP e) = pp e
 pp (JUMP j js) = text "JUMP" <+> pp j -- <+> lbrack <> pp js <> rbrack
 pp (CJUMP op l r tL eL) = text "CJUMP" <+> (parens (pp l <> pp op <> pp r)) <+> ppString tL <+> colon <+> ppString eL
 pp (SEQ x xs) = pp x $$ pp xs
 pp (LABEL l) = ppString l

instance Renderable BinOp where
 pp  AbsSyn.PLUS = text "+"
 pp  AbsSyn.MINUS = text "-"
 pp  AbsSyn.MUL = text "*"
 pp  AbsSyn.DIV = text "/"
 pp  AbsSyn.AND = text "&&"
 pp  AbsSyn.OR = text "||"
 pp  AbsSyn.LSHIFT = text "<<"
 pp  AbsSyn.RSHIFT = text ">>"
 pp  AbsSyn.ARSHIFT = text "<<#"
 pp  AbsSyn.XOR = text "^^"


instance Renderable RelOp where
 pp  AbsSyn.EQ = text "=="
 pp  AbsSyn.NE = text "!="
 pp  AbsSyn.LT = text "<"
 pp  AbsSyn.GT = text ">"
 pp  AbsSyn.LE = text "<="
 pp  AbsSyn.GE = text ">="
 pp  AbsSyn.ULT = text "<#"
 pp  AbsSyn.ULE = text "<=#"
 pp  AbsSyn.UGT = text ">#"
 pp  AbsSyn.UGE = text ">#"

{-
instance Renderable [Expression] where
 pp ss = punctuate "," ss
-}

instance Renderable [Exp] where
 pp ss = hcat (map pp ss)

instance Renderable [Stm] where
 pp ss = vcat (map pp ss)

instance Renderable Char where
 pp c = text (show c)

instance Renderable Int where
 pp n = int n

ppString str = text str